/*
  functions.cc: mathematical functions
  Copyright 2015 by CNRS/AMU

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <headers.hh>
#include <functions.hh>

#include <gsl/gsl_randist.h>
#include <gsl/gsl_sf.h>

// atan:
//
//             3        5        7        9      10
//    x - 1/3 x  + 1/5 x  - 1/7 x  + 1/9 x  + O(x  )
double Functions::atanc(double x)
{
  if(x < 0.01 && x > -0.01)
    return 1 - x*x/3 + gsl_pow_4(x)/5 - gsl_pow_6(x)/7 + gsl_pow_8(x)/9;
  else
    return atan(x)/x;
}

//
// atanh:    
//             3        5        7        9      10
//    x + 1/3 x  + 1/5 x  + 1/7 x  + 1/9 x  + O(x  )
double Functions::atanhc(double x)
{
  if(x < 0.01 && x > -0.01)
    return 1 + x*x/3 + gsl_pow_4(x)/5 + gsl_pow_6(x)/7 + gsl_pow_8(x)/9;
  else
    return atanh(x)/x;
}

double Functions::pseudoVoigt(double x, double w, double mu)
{
  return (1 - mu) * gsl_ran_gaussian_pdf(x, w) + mu *
    gsl_ran_cauchy_pdf(x, w);
}

double Functions::marcusHushChidseyZeng(double lambda, double eta)
{
  double a = 1 + sqrt(lambda);
  return sqrt(M_PI * lambda)/(1 + exp(- eta)) * gsl_sf_erfc((lambda - sqrt(a + eta*eta))/(2 * sqrt(lambda)));
}
